/*
 * Copyright E. Bigeon (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * Socket implementation of GCLC.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
package fr.bigeon.gclc.socket;

import java.io.IOException;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.command.ExitCommand;
import fr.bigeon.gclc.command.HelpExecutor;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.InvalidCommandName;
import fr.bigeon.gclc.manager.ConsoleManager;

/** A test-purpose application
 * 
 * @author Emmanuel Bigeon */
public class ConsoleTestApplication {

    /** Exit command */
    public static final String EXIT = "exit"; //$NON-NLS-1$

    /** @param manager the manager
     * @return create the application */
    @SuppressWarnings("nls")
    public static ConsoleApplication create(final ConsoleManager manager) {
        try {
            ConsoleApplication application = new ConsoleApplication(manager,
                    "Welcome to the test application. Type help or test.",
                    "See you");
            application.add(new ExitCommand(EXIT, application));
            application.add(
                    new HelpExecutor("help", manager, application.root));
            application.add(new Command("test") {

                @Override
                public String tip() {
                    return "A test command";
                }

                @Override
                public void execute(String... args) throws CommandRunException {
                    try {
                        manager.println("Test command ran fine");
                    } catch (IOException e) {
                        throw new CommandRunException("manager closed", e,
                                this);
                    }
                }
            });
            application.add(new Command("long") {

                @Override
                public String tip() {
                    return "A long run test command";
                }

                @Override
                public void execute(String... args) throws CommandRunException {
                    try {
                        Thread.sleep(2000);
                        manager.println("Test command ran fine");
                    } catch (IOException | InterruptedException e) {
                        throw new CommandRunException("manager closed", e,
                                this);
                    }
                }
            });
            return application;
        } catch (final InvalidCommandName e) {
            e.printStackTrace();
        }
        return null;
    }
}
