/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.prompt.CLIPrompterTest.java
 * Created on: Nov 18, 2016
 */
package fr.bigeon.gclc.prompt;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import fr.bigeon.gclc.manager.PipedConsoleInput;
import fr.bigeon.gclc.manager.PipedConsoleOutput;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class CLIPrompterTest {

    /** @throws java.lang.Exception if there are exceptions */
    @Before
    public void setUp() throws Exception {
        //
    }

    /** @throws java.lang.Exception if there are exceptions */
    @After
    public void tearDown() throws Exception {
        //
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptBoolean(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String)}. */
    @Test
    public final void testPromptBoolean() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertTrue(CLIPrompter.promptBoolean(out, in,
                                "My message")); //$NON-NLS-1$
                        assertTrue(CLIPrompter.promptBoolean(out, in,
                                "My message")); //$NON-NLS-1$
                        assertFalse(CLIPrompter.promptBoolean(out, in,
                                "My message")); //$NON-NLS-1$
                        assertFalse(CLIPrompter.promptBoolean(out, in,
                                "My message")); //$NON-NLS-1$
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(in.readNextLine().startsWith("My message")); //$NON-NLS-1$
            in.type(""); //$NON-NLS-1$
            out.readNextLine();
            assertTrue(in.readNextLine().startsWith("My message")); //$NON-NLS-1$
            in.type("Y"); //$NON-NLS-1$
            assertTrue(in.readNextLine().startsWith("My message")); //$NON-NLS-1$
            in.type("yes"); //$NON-NLS-1$
            assertTrue(in.readNextLine().startsWith("My message")); //$NON-NLS-1$
            in.type("N"); //$NON-NLS-1$
            assertTrue(in.readNextLine().startsWith("My message")); //$NON-NLS-1$
            in.type("nO"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }

    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptChoice(fr.bigeon.gclc.manager.ConsoleManager, java.util.List, java.util.List, java.lang.String, java.lang.String)}. */
    @Test
    public final void testPromptChoiceConsoleManagerListOfStringListOfUStringString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<String> keys = new ArrayList<>();
            final List<Object> choices = new ArrayList<>();
            keys.add("A choice"); //$NON-NLS-1$
            choices.add(new Integer(25));
            keys.add("An other"); //$NON-NLS-1$
            choices.add("The actual other"); //$NON-NLS-1$
            final String cancel = "Cancel"; //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals(choices.get(0), CLIPrompter.promptChoice(
                                out, in, keys, choices, message, cancel));
                        assertEquals(choices.get(0), CLIPrompter.promptChoice(
                                out, in, keys, choices, message, null));
                        assertEquals(null, CLIPrompter.promptChoice(out, in,
                                keys, choices, message, cancel));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("yoyo"); //$NON-NLS-1$
            // fail, reprompt
            final String msg = CLIPrompterMessages.getString(
                    "promptchoice.formaterr", //$NON-NLS-1$
                    0, keys.size());
            for (final String line : msg.split(System.lineSeparator())) {
                assertEquals(line, out.readNextLine());
            }
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, reprompt without cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            // fail, reprompt
            assertEquals(
                    CLIPrompterMessages.getString("promptchoice.outofbounds", 0, //$NON-NLS-1$
                            keys.size() - 1),
                    out.readNextLine());
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, prompt with cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    @Test
    public final void testPromptChoiceConsoleManagerListOfUMapOfUTStringString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<Object> keys = new ArrayList<>();
            final Map<Object, Object> choices = new HashMap<>();
            keys.add("A choice"); //$NON-NLS-1$
            choices.put(keys.get(0), new Integer(25));
            keys.add("An other"); //$NON-NLS-1$
            choices.put(keys.get(1), "The actual other"); //$NON-NLS-1$
            final String cancel = "Cancel"; //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals(choices.get(keys.get(0)),
                                CLIPrompter.promptChoice(out, in, keys,
                                        choices, message, cancel));
                        assertEquals(choices.get(keys.get(0)),
                                CLIPrompter.promptChoice(out, in, keys,
                                        choices, message, null));
                        assertEquals(null, CLIPrompter.promptChoice(out, in,
                                keys, choices, message, cancel));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("yoyo"); //$NON-NLS-1$
            // fail, reprompt
            final String msg = CLIPrompterMessages.getString(
                    "promptchoice.formaterr", //$NON-NLS-1$
                    0, keys.size());
            for (final String line : msg.split(System.lineSeparator())) {
                assertEquals(line, out.readNextLine());
            }
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, reprompt without cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            // fail, reprompt
            assertEquals(
                    CLIPrompterMessages.getString("promptchoice.outofbounds", 0, //$NON-NLS-1$
                            keys.size() - 1),
                    out.readNextLine());
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, prompt with cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    @Test
    public final void testPromptChoiceConsoleManagerListOfUStringString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<Object> keys = new ArrayList<>();
            keys.add("A choice"); //$NON-NLS-1$
            keys.add("An other"); //$NON-NLS-1$
            final String cancel = "Cancel"; //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals(Integer.valueOf(0),
                                CLIPrompter.promptChoice(out, in, keys,
                                        message, cancel));
                        assertEquals(Integer.valueOf(0), CLIPrompter
                                .promptChoice(out, in, keys, message, null));
                        assertEquals(Integer.valueOf(1), CLIPrompter
                                .promptChoice(out, in, keys, message, null));
                        assertEquals(null, CLIPrompter.promptChoice(out, in,
                                keys, message, cancel));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("yoyo"); //$NON-NLS-1$
            // fail, reprompt
            final String msg = CLIPrompterMessages.getString(
                    "promptchoice.formaterr", //$NON-NLS-1$
                    0, keys.size());
            for (final String line : msg.split(System.lineSeparator())) {
                assertEquals(line, out.readNextLine());
            }
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, reprompt without cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            // fail, reprompt
            assertEquals(
                    CLIPrompterMessages.getString("promptchoice.outofbounds", 0, //$NON-NLS-1$
                            keys.size() - 1),
                    out.readNextLine());
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Success do it again
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("1"); //$NON-NLS-1$
            // Sucess, prompt with cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptChoice(fr.bigeon.gclc.manager.ConsoleManager, java.util.Map, java.lang.String, java.lang.String)}. */
    @Test
    public final void testPromptChoiceConsoleManagerMapOfUTStringString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<Object> keys = new ArrayList<>();
            final Map<Object, Object> choices = new HashMap<>();
            keys.add("A choice"); //$NON-NLS-1$
            choices.put(keys.get(0), new Integer(25));
            keys.add("An other"); //$NON-NLS-1$
            choices.put(keys.get(1), "The actual other"); //$NON-NLS-1$
            final String cancel = "Cancel"; //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals(choices.get(keys.get(0)),
                                CLIPrompter.promptChoice(out, in, keys,
                                        choices, message, cancel));
                        assertEquals(choices.get(keys.get(0)),
                                CLIPrompter.promptChoice(out, in, keys,
                                        choices, message, null));
                        assertEquals(null, CLIPrompter.promptChoice(out, in,
                                keys, choices, message, cancel));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(out.readNextLine().startsWith(message));
            final String readNextLine = out.readNextLine();
            assertTrue(readNextLine + " expected to contain " + keys.get(0),
                    readNextLine.contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("yoyo"); //$NON-NLS-1$
            // fail, reprompt
            final String msg = CLIPrompterMessages.getString(
                    "promptchoice.formaterr", //$NON-NLS-1$
                    0, keys.size());
            for (final String line : msg.split(System.lineSeparator())) {
                assertEquals(line, out.readNextLine());
            }
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, reprompt without cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            // fail, reprompt
            assertEquals(
                    CLIPrompterMessages.getString("promptchoice.outofbounds", 0, //$NON-NLS-1$
                            keys.size() - 1),
                    out.readNextLine());
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, prompt with cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertTrue(out.readNextLine().contains(cancel));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptInteger(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String)}. */
    @Test
    public final void testPromptInteger() {
        try (final PipedConsoleInput test = new PipedConsoleInput()) {
            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals(10,
                                CLIPrompter.promptInteger(test, "My message")); //$NON-NLS-1$
                        assertEquals(-15,
                                CLIPrompter.promptInteger(test, "My message")); //$NON-NLS-1$
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(test.readNextLine().startsWith("My message")); //$NON-NLS-1$
            test.type(""); //$NON-NLS-1$
            assertTrue(test.readNextLine().startsWith("My message")); //$NON-NLS-1$
            test.type("Y"); //$NON-NLS-1$
            assertTrue(test.readNextLine().startsWith("My message")); //$NON-NLS-1$
            test.type("10"); //$NON-NLS-1$
            assertTrue(test.readNextLine().startsWith("My message")); //$NON-NLS-1$
            test.type("-15"); //$NON-NLS-1$
            th.join();

        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptList(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String)}. */
    @Test
    public final void testPromptListConsoleManagerString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<String> keys = new ArrayList<>();
            keys.add("A choice"); //$NON-NLS-1$
            keys.add("An other"); //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals(new ArrayList<String>(),
                                CLIPrompter.promptList(out, in, message));
                        assertEquals(keys,
                                CLIPrompter.promptList(out, in, message));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            String nLine = out.readNextLine();
            assertTrue(nLine.startsWith(message));
            assertTrue(nLine.endsWith(CLIPrompterMessages
                    .getString("promptlist.exit.dispkey", CLIPrompterMessages //$NON-NLS-1$
                            .getString("promptlist.exit.defaultkey")))); //$NON-NLS-1$
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type(CLIPrompterMessages
                    .getString("promptlist.exit.defaultkey")); //$NON-NLS-1$
            // enter keys list
            nLine = out.readNextLine();
            assertTrue(nLine.startsWith(message));
            assertTrue(nLine.endsWith(CLIPrompterMessages
                    .getString("promptlist.exit.dispkey", CLIPrompterMessages //$NON-NLS-1$
                            .getString("promptlist.exit.defaultkey")))); //$NON-NLS-1$
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            for (int i = 0; i < keys.size(); i++) {
                in.type(keys.get(i));
                assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                        in.readNextLine());
            }
            in.type(CLIPrompterMessages
                    .getString("promptlist.exit.defaultkey")); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptList(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String, java.lang.String)}. */
    @Test
    public final void testPromptListConsoleManagerStringString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<String> keys = new ArrayList<>();
            keys.add("A choice"); //$NON-NLS-1$
            keys.add("An other"); //$NON-NLS-1$
            final String ender = "*"; //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals(new ArrayList<String>(), CLIPrompter
                                .promptList(out, in, message, ender));
                        assertEquals(keys, CLIPrompter.promptList(out, in,
                                message, ender));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            String nLine = out.readNextLine();
            assertTrue(nLine.startsWith(message));
            assertTrue(nLine.endsWith(CLIPrompterMessages
                    .getString("promptlist.exit.dispkey", ender))); //$NON-NLS-1$
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type(ender);
            // enter keys list
            nLine = out.readNextLine();
            assertTrue(nLine.startsWith(message));
            assertTrue(nLine.endsWith(CLIPrompterMessages
                    .getString("promptlist.exit.dispkey", ender))); //$NON-NLS-1$
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            for (int i = 0; i < keys.size(); i++) {
                in.type(keys.get(i));
                assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                        in.readNextLine());
            }
            in.type(ender);
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptLongText(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String)}. */
    @Test
    public final void testPromptLongTextConsoleManagerString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final String message = "My message";
            final String longText = "Some text with" + System.lineSeparator() +
                                    "line feeds and other" +
                                    System.lineSeparator() +
                                    System.lineSeparator() +
                                    " \tspecial characters";

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals("", CLIPrompter.promptLongText(out, in,
                                message));
                        assertEquals(longText + System.lineSeparator(),
                                CLIPrompter.promptLongText(out, in,
                                        message));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            String nLine = out.readNextLine();
            assertTrue(nLine.startsWith(message));
            assertTrue(nLine.endsWith(CLIPrompterMessages.getString(
                    "promptlongtext.exit.dispkey", CLIPrompterMessages
                            .getString("promptlongtext.exit.defaultkey"))));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"),
                    in.readNextLine());
            in.type(CLIPrompterMessages
                    .getString("promptlongtext.exit.defaultkey"));
            // enter long text
            nLine = out.readNextLine();
            assertTrue(nLine.startsWith(message));
            assertTrue(nLine.endsWith(CLIPrompterMessages.getString(
                    "promptlongtext.exit.dispkey", CLIPrompterMessages
                            .getString("promptlongtext.exit.defaultkey"))));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"),
                    in.readNextLine());
            final String[] text = longText.split(System.lineSeparator());
            for (final String element : text) {
                in.type(element);
                assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"),
                        in.readNextLine());
            }
            in.type(CLIPrompterMessages
                    .getString("promptlongtext.exit.defaultkey"));
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptLongText(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String, java.lang.String)}. */
    @Test
    public final void testPromptLongTextConsoleManagerStringString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final String message = "My message";
            final String ender = "\\quit";
            final String[] text = new String[] {"Some text with",
                    "feeds and other", " \tspecial characters"};
            final String longText = text[0] + System.lineSeparator() + text[1] +
                                    System.lineSeparator() + text[2];

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals("", //$NON-NLS-1$
                                CLIPrompter.promptLongText(out, in, message,
                                        ender));
                        assertEquals(longText + System.lineSeparator(),
                                CLIPrompter.promptLongText(out, in, message,
                                        ender));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            String nLine = out.readNextLine();
            assertTrue(nLine.startsWith(message));
            assertTrue(nLine.endsWith(CLIPrompterMessages
                    .getString("promptlongtext.exit.dispkey", ender)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"),
                    in.readNextLine());
            in.type(ender);
            // enter long text
            nLine = out.readNextLine();
            assertTrue(nLine.startsWith(message));
            assertTrue(nLine.endsWith(CLIPrompterMessages
                    .getString("promptlongtext.exit.dispkey", ender)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"),
                    in.readNextLine());
            for (final String element : text) {
                in.type(element);
                assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"),
                        in.readNextLine());
            }
            in.type(ender);
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptMultiChoice(fr.bigeon.gclc.manager.ConsoleManager, java.util.List, java.util.List, java.lang.String)}. */
    @Test
    public final void testPromptMultiChoiceConsoleManagerListOfStringListOfUString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<String> keys = new ArrayList<>();
            final List<Object> choices = new ArrayList<>();
            keys.add("A choice"); //$NON-NLS-1$
            choices.add(new Integer(25));
            keys.add("An other"); //$NON-NLS-1$
            choices.add("The actual other"); //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertTrue(CLIPrompter.promptMultiChoice(out, in,
                                keys, choices, message).isEmpty());
                        final ArrayList<Object> l = new ArrayList<>();
                        l.add(choices.get(0));
                        assertEquals(l, CLIPrompter.promptMultiChoice(out, in,
                                keys, choices, message));
                        l.add(choices.get(1));
                        assertEquals(l, CLIPrompter.promptMultiChoice(out, in,
                                keys, choices, message));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("yoyo"); //$NON-NLS-1$
            // fail, reprompt
            final String msg = CLIPrompterMessages.getString(
                    "promptchoice.formaterr", //$NON-NLS-1$
                    0, keys.size() - 1);
            for (final String line : msg.split(System.lineSeparator())) {
                assertEquals(line, out.readNextLine());
            }
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type(""); //$NON-NLS-1$
            // Sucess, reprompt without cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            // fail, reprompt
            assertEquals(
                    CLIPrompterMessages.getString("promptchoice.outofbounds", 0, //$NON-NLS-1$
                            keys.size() - 1),
                    out.readNextLine());
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, prompt with cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0)));
            assertTrue(out.readNextLine().contains(keys.get(1)));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0 1"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptMultiChoice(fr.bigeon.gclc.manager.ConsoleManager, java.util.List, java.util.Map, java.lang.String)}. */
    @Test
    public final void testPromptMultiChoiceConsoleManagerListOfUMapOfUTString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<Object> keys = new ArrayList<>();
            final Map<Object, Object> choices = new HashMap<>();
            keys.add("A choice"); //$NON-NLS-1$
            choices.put(keys.get(0), new Integer(25));
            keys.add("An other"); //$NON-NLS-1$
            choices.put(keys.get(1), "The actual other"); //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertTrue(CLIPrompter.promptMultiChoice(out, in,
                                keys, choices, message).isEmpty());
                        final ArrayList<Object> l = new ArrayList<>();
                        l.add(choices.get(keys.get(0)));
                        assertEquals(l, CLIPrompter.promptMultiChoice(out, in,
                                keys, choices, message));
                        l.add(choices.get(keys.get(1)));
                        assertEquals(l, CLIPrompter.promptMultiChoice(out, in,
                                keys, choices, message));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("yoyo"); //$NON-NLS-1$
            // fail, reprompt
            final String msg = CLIPrompterMessages.getString(
                    "promptchoice.formaterr", //$NON-NLS-1$
                    0, keys.size() - 1);
            for (final String line : msg.split(System.lineSeparator())) {
                assertEquals(line, out.readNextLine());
            }
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type(""); //$NON-NLS-1$
            // Sucess, reprompt without cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            // fail, reprompt
            assertEquals(
                    CLIPrompterMessages.getString("promptchoice.outofbounds", 0, //$NON-NLS-1$
                            keys.size() - 1),
                    out.readNextLine());
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, prompt with cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0 1"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptMultiChoice(fr.bigeon.gclc.manager.ConsoleManager, java.util.List, java.lang.String)}. */
    @Test
    public final void testPromptMultiChoiceConsoleManagerListOfUString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<Object> keys = new ArrayList<>();
            keys.add("A choice"); //$NON-NLS-1$
            keys.add("An other"); //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertTrue(CLIPrompter
                                .promptMultiChoice(out, in, keys, message)
                                .isEmpty());
                        final ArrayList<Integer> l = new ArrayList<>();
                        l.add(0);
                        assertEquals(l, CLIPrompter.promptMultiChoice(out, in,
                                keys, message));
                        l.add(1);
                        assertEquals(l, CLIPrompter.promptMultiChoice(out, in,
                                keys, message));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("yoyo"); //$NON-NLS-1$
            // fail, reprompt
            final String msg = CLIPrompterMessages.getString(
                    "promptchoice.formaterr", //$NON-NLS-1$
                    0, keys.size() - 1);
            for (final String line : msg.split(System.lineSeparator())) {
                assertEquals(line, out.readNextLine());
            }
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type(""); //$NON-NLS-1$
            // Sucess, reprompt without cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("2"); //$NON-NLS-1$
            // fail, reprompt
            assertEquals(
                    CLIPrompterMessages.getString("promptchoice.outofbounds", 0, //$NON-NLS-1$
                            keys.size() - 1),
                    out.readNextLine());
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0"); //$NON-NLS-1$
            // Sucess, prompt with cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            assertEquals(CLIPrompterMessages.getString("prompt.lineprompt"), //$NON-NLS-1$
                    in.readNextLine());
            in.type("0 1"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptMultiChoice(fr.bigeon.gclc.manager.ConsoleManager, java.util.Map, java.lang.String)}. */
    @Test
    public final void testPromptMultiChoiceConsoleManagerMapOfUTString() {
        try (final PipedConsoleOutput out = new PipedConsoleOutput();
             PipedConsoleInput in = new PipedConsoleInput()) {
            final List<Object> keys = new ArrayList<>();
            final Map<Object, Object> choices = new HashMap<>();
            keys.add("A choice"); //$NON-NLS-1$
            choices.put(keys.get(0), new Integer(25));
            keys.add("An other"); //$NON-NLS-1$
            choices.put(keys.get(1), "The actual other"); //$NON-NLS-1$
            final String message = "My message"; //$NON-NLS-1$

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertTrue(CLIPrompter.promptMultiChoice(out, in, keys,
                                choices, message).isEmpty());
                        final ArrayList<Object> l = new ArrayList<>();
                        l.add(choices.get(keys.get(0)));
                        assertEquals(l, CLIPrompter.promptMultiChoice(out, in,
                                keys, choices, message));
                        l.add(choices.get(keys.get(1)));
                        assertEquals(l, CLIPrompter.promptMultiChoice(out, in,
                                keys, choices, message));
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(out.readNextLine().startsWith(message));
            final String readNextLine = out.readNextLine();
            assertTrue(readNextLine + " expected to contain " + keys.get(0),
                    readNextLine.contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            in.type("yoyo"); //$NON-NLS-1$
            // fail, reprompt
            final String msg = CLIPrompterMessages.getString(
                    "promptchoice.formaterr", //$NON-NLS-1$
                    0, keys.size() - 1);
            for (final String line : msg.split(System.lineSeparator())) {
                assertEquals(line, out.readNextLine());
            }
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            in.type(""); //$NON-NLS-1$
            // Sucess, reprompt without cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            in.type("2"); //$NON-NLS-1$
            // fail, reprompt
            assertEquals(
                    CLIPrompterMessages.getString("promptchoice.outofbounds", 0, //$NON-NLS-1$
                            keys.size() - 1),
                    out.readNextLine());
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            in.type("0"); //$NON-NLS-1$
            // Sucess, prompt with cancel
            assertTrue(out.readNextLine().startsWith(message));
            assertTrue(out.readNextLine().contains(keys.get(0).toString()));
            assertTrue(out.readNextLine().contains(keys.get(1).toString()));
            in.type("0 1"); //$NON-NLS-1$
            th.join();
        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion, the other thread probably failed"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.prompt.CLIPrompter#promptNonEmpty(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String, java.lang.String)}. */
    @Test
    public final void testPromptNonEmpty() {
        try (final PipedConsoleInput test = new PipedConsoleInput()) {
            final String res = "some content"; //$NON-NLS-1$
            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        assertEquals(res, CLIPrompter.promptNonEmpty(test,
                                "My message", "my reprompt")); //$NON-NLS-1$ //$NON-NLS-2$
                    } catch (final IOException e) {
                        fail("Unexpected io excpetion"); //$NON-NLS-1$
                        e.printStackTrace();
                    }
                }
            });
            th.start();
            assertTrue(test.readNextLine().startsWith("My message")); //$NON-NLS-1$
            test.type(""); //$NON-NLS-1$
            assertTrue(test.readNextLine().startsWith("my reprompt")); //$NON-NLS-1$
            test.type(res);
            th.join();

        } catch (IOException | InterruptedException e) {
            fail("Unexpected excpetion"); //$NON-NLS-1$
            e.printStackTrace();
        }
    }

}
